import { nativeTimeout, rafFrames } from '../../util/dom';
import { isActivatedDisabled } from './activator-base';
export var Activator = (function () {
    function Activator(app, config) {
        this.app = app;
        this._queue = [];
        this._active = [];
        this.activatedDelay = ADD_ACTIVATED_DEFERS;
        this.clearDelay = CLEAR_STATE_DEFERS;
        this._css = config.get('activatedClass') || 'activated';
    }
    Activator.prototype.clickAction = function (ev, activatableEle, startCoord) {
        this._scheduleClear();
        this._queue.length = 0;
        for (var i = 0; i < this._active.length; i++) {
            this._active[i].classList.remove(this._css);
        }
        this._active.length = 0;
        if (activatableEle && activatableEle.parentNode) {
            this._active.push(activatableEle);
            activatableEle.classList.add(this._css);
        }
    };
    Activator.prototype.downAction = function (ev, activatableEle, startCoord) {
        var _this = this;
        if (isActivatedDisabled(ev, activatableEle)) {
            return;
        }
        this.unscheduleClear();
        this.deactivate();
        this._queue.push(activatableEle);
        this._activeRafDefer = rafFrames(this.activatedDelay, function () {
            var activatableEle;
            for (var i = 0; i < _this._queue.length; i++) {
                activatableEle = _this._queue[i];
                _this._active.push(activatableEle);
                activatableEle.classList.add(_this._css);
            }
            _this._queue.length = 0;
            _this._clearDeferred();
        });
    };
    Activator.prototype.upAction = function (ev, activatableEle, startCoord) {
        this._scheduleClear();
    };
    Activator.prototype._scheduleClear = function () {
        var _this = this;
        if (this._clearRafDefer) {
            return;
        }
        this._clearRafDefer = rafFrames(this.clearDelay, function () {
            _this.clearState();
            _this._clearRafDefer = null;
        });
    };
    Activator.prototype.unscheduleClear = function () {
        if (this._clearRafDefer) {
            this._clearRafDefer();
            this._clearRafDefer = null;
        }
    };
    Activator.prototype.clearState = function () {
        var _this = this;
        if (!this.app.isEnabled()) {
            nativeTimeout(function () {
                _this.clearState();
            }, 600);
        }
        else {
            this.deactivate();
        }
    };
    Activator.prototype.deactivate = function () {
        this._clearDeferred();
        this._queue.length = 0;
        for (var i = 0; i < this._active.length; i++) {
            this._active[i].classList.remove(this._css);
        }
        this._active.length = 0;
    };
    Activator.prototype._clearDeferred = function () {
        if (this._activeRafDefer) {
            this._activeRafDefer();
            this._activeRafDefer = null;
        }
    };
    return Activator;
}());
var ADD_ACTIVATED_DEFERS = 6;
var CLEAR_STATE_DEFERS = 6;
//# sourceMappingURL=activator.js.map