(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '@angular/core', './virtual-util', '../../config/config', '../content/content', '../../util/dom-controller', '../../util/util', '../../platform/platform', '../../navigation/view-controller', './virtual-item'], factory);
    }
})(function (require, exports) {
    "use strict";
    var core_1 = require('@angular/core');
    var virtual_util_1 = require('./virtual-util');
    var config_1 = require('../../config/config');
    var content_1 = require('../content/content');
    var dom_controller_1 = require('../../util/dom-controller');
    var util_1 = require('../../util/util');
    var platform_1 = require('../../platform/platform');
    var view_controller_1 = require('../../navigation/view-controller');
    var virtual_item_1 = require('./virtual-item');
    var VirtualScroll = (function () {
        function VirtualScroll(_iterableDiffers, _elementRef, _renderer, _zone, _cd, _content, _platform, _ctrl, config, _dom) {
            var _this = this;
            this._iterableDiffers = _iterableDiffers;
            this._elementRef = _elementRef;
            this._renderer = _renderer;
            this._zone = _zone;
            this._cd = _cd;
            this._content = _content;
            this._platform = _platform;
            this._ctrl = _ctrl;
            this._dom = _dom;
            this._records = [];
            this._cells = [];
            this._nodes = [];
            this._vHeight = 0;
            this._lastCheck = 0;
            this._data = {
                scrollTop: 0,
            };
            this._queue = null;
            this.bufferRatio = 2;
            this.approxItemWidth = '100%';
            this.approxHeaderWidth = '100%';
            this.approxHeaderHeight = '40px';
            this.approxFooterWidth = '100%';
            this.approxFooterHeight = '40px';
            this._hasUpdate = false;
            this._renderer.setElementClass(_elementRef.nativeElement, 'virtual-loading', true);
            _content.readReady.subscribe(function () {
                _this.readUpdate(true, true);
                if (!_this._scrollSub) {
                    _this.addScrollListener(config.getBoolean('virtualScrollEventAssist'));
                }
            });
            _content.writeReady.subscribe(function () {
                _this.writeUpdate();
            });
        }
        Object.defineProperty(VirtualScroll.prototype, "virtualScroll", {
            set: function (val) {
                this._records = val;
                if (util_1.isBlank(this._differ) && util_1.isPresent(val)) {
                    this._differ = this._iterableDiffers.find(val).create(this._cd, this._trackBy);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VirtualScroll.prototype, "headerFn", {
            set: function (val) {
                if (util_1.isFunction(val)) {
                    this._hdrFn = val.bind((this._ctrl && this._ctrl._cmp) || this);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VirtualScroll.prototype, "footerFn", {
            set: function (val) {
                if (util_1.isFunction(val)) {
                    this._ftrFn = val.bind((this._ctrl && this._ctrl._cmp) || this);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VirtualScroll.prototype, "virtualTrackBy", {
            set: function (val) {
                this._trackBy = val;
            },
            enumerable: true,
            configurable: true
        });
        VirtualScroll.prototype.readUpdate = function (checkDataChanges, dimensionsUpdated) {
            if (!this._records)
                return;
            if (checkDataChanges && !dimensionsUpdated) {
                if (util_1.isPresent(this._differ) && !util_1.isPresent(this._differ.diff(this._records))) {
                    return;
                }
            }
            (void 0);
            this._hasUpdate = true;
            this._cells.length = 0;
            this._nodes.length = 0;
            this._itmTmp.viewContainer.clear();
            virtual_util_1.calcDimensions(this._data, this._elementRef.nativeElement, this.approxItemWidth, this.approxItemHeight, this.approxHeaderWidth, this.approxHeaderHeight, this.approxFooterWidth, this.approxFooterHeight, this.bufferRatio);
        };
        VirtualScroll.prototype.writeUpdate = function () {
            if (!this._hasUpdate) {
                return;
            }
            (void 0);
            virtual_util_1.processRecords(this._data.renderHeight, this._records, this._cells, this._hdrFn, this._ftrFn, this._data);
            this.renderVirtual();
            this._hasUpdate = false;
        };
        VirtualScroll.prototype.ngDoCheck = function () {
            if (this._init) {
                this.readUpdate(true, false);
                this.writeUpdate();
            }
        };
        VirtualScroll.prototype.ngAfterContentInit = function () {
            if (!this._init) {
                if (!this._itmTmp) {
                    throw 'virtualItem required within virtualScroll';
                }
                this._init = true;
                if (!this.approxItemHeight) {
                    this.approxItemHeight = '40px';
                    console.warn('Virtual Scroll: Please provide an "approxItemHeight" input to ensure proper virtual scroll rendering');
                }
            }
        };
        VirtualScroll.prototype.renderVirtual = function () {
            var _this = this;
            var nodes = this._nodes;
            var cells = this._cells;
            var data = this._data;
            var records = this._records;
            data.topCell = 0;
            data.bottomCell = (cells.length - 1);
            virtual_util_1.populateNodeData(0, data.bottomCell, data.viewWidth, true, cells, records, nodes, this._itmTmp.viewContainer, this._itmTmp.templateRef, this._hdrTmp && this._hdrTmp.templateRef, this._ftrTmp && this._ftrTmp.templateRef, true);
            this._cd.detectChanges();
            this._dom.read(function () {
                virtual_util_1.initReadNodes(nodes, cells, data);
            });
            this._dom.write(function () {
                var ele = _this._elementRef.nativeElement;
                var recordsLength = records.length;
                var renderer = _this._renderer;
                virtual_util_1.updateNodeContext(nodes, cells, data);
                _this._cd.detectChanges();
                var lastEle = renderer.createElement(ele, 'div');
                lastEle.className = 'virtual-last';
                renderer.setElementClass(ele, 'virtual-scroll', true);
                renderer.setElementClass(ele, 'virtual-loading', false);
                virtual_util_1.writeToNodes(nodes, cells, recordsLength);
                _this.setVirtualHeight(virtual_util_1.estimateHeight(recordsLength, cells[cells.length - 1], _this._vHeight, 0.25));
                _this._content.imgsRefresh();
            });
        };
        VirtualScroll.prototype.scrollUpdate = function (ev) {
            var _this = this;
            var data = this._data;
            var cells = this._cells;
            var nodes = this._nodes;
            data.scrollTop = ev.scrollTop;
            if (this._queue === 2) {
                this._dom.write(function () {
                    virtual_util_1.writeToNodes(nodes, cells, _this._records.length);
                    _this.setVirtualHeight(virtual_util_1.estimateHeight(_this._records.length, cells[cells.length - 1], _this._vHeight, 0.25));
                    _this._queue = 0;
                });
            }
            else if (this._queue === 1) {
                this._dom.write(function () {
                    for (var i = 0; i < nodes.length; i++) {
                        if (nodes[i].hasChanges) {
                            nodes[i].view.detectChanges();
                        }
                    }
                    _this._queue = 2;
                });
            }
            else {
                data.scrollDiff = (data.scrollTop - this._lastCheck);
                if (Math.abs(data.scrollDiff) > SCROLL_DIFFERENCE_MINIMUM) {
                    this._lastCheck = data.scrollTop;
                    if (data.scrollDiff > 0) {
                        var stopAtHeight = (data.scrollTop + data.renderHeight);
                        virtual_util_1.processRecords(stopAtHeight, this._records, cells, this._hdrFn, this._ftrFn, data);
                    }
                    virtual_util_1.updateDimensions(nodes, cells, data, false);
                    virtual_util_1.adjustRendered(cells, data);
                    var hasChanges = virtual_util_1.populateNodeData(data.topCell, data.bottomCell, data.viewWidth, data.scrollDiff > 0, cells, this._records, nodes, this._itmTmp.viewContainer, this._itmTmp.templateRef, this._hdrTmp && this._hdrTmp.templateRef, this._ftrTmp && this._ftrTmp.templateRef, false);
                    if (hasChanges) {
                        this._queue = 1;
                        virtual_util_1.updateNodeContext(nodes, cells, data);
                    }
                }
            }
        };
        VirtualScroll.prototype.scrollEnd = function (ev) {
            var _this = this;
            var nodes = this._nodes;
            var cells = this._cells;
            var data = this._data;
            virtual_util_1.updateDimensions(nodes, cells, data, false);
            virtual_util_1.adjustRendered(cells, data);
            this._dom.write(function () {
                virtual_util_1.updateNodeContext(nodes, cells, data);
                _this._cd.detectChanges();
                virtual_util_1.writeToNodes(nodes, cells, _this._records.length);
                _this.setVirtualHeight(virtual_util_1.estimateHeight(_this._records.length, cells[cells.length - 1], _this._vHeight, 0.05));
                _this._queue = 0;
            });
        };
        VirtualScroll.prototype.setVirtualHeight = function (newVirtualHeight) {
            if (newVirtualHeight !== this._vHeight) {
                this._renderer.setElementStyle(this._elementRef.nativeElement, 'height', newVirtualHeight > 0 ? newVirtualHeight + 'px' : '');
                this._vHeight = newVirtualHeight;
                (void 0);
            }
        };
        VirtualScroll.prototype.addScrollListener = function (eventAssist) {
            var _this = this;
            if (eventAssist) {
                this._content.enableJsScroll();
            }
            this._scrollSub = this._content.ionScroll.subscribe(function (ev) {
                _this.scrollUpdate(ev);
            });
            this._scrollEndSub = this._content.ionScrollEnd.subscribe(function (ev) {
                _this.scrollEnd(ev);
            });
        };
        VirtualScroll.prototype.ngOnDestroy = function () {
            this._scrollSub && this._scrollSub.unsubscribe();
            this._scrollEndSub && this._scrollEndSub.unsubscribe();
        };
        VirtualScroll.decorators = [
            { type: core_1.Directive, args: [{
                        selector: '[virtualScroll]'
                    },] },
        ];
        VirtualScroll.ctorParameters = [
            { type: core_1.IterableDiffers, },
            { type: core_1.ElementRef, },
            { type: core_1.Renderer, },
            { type: core_1.NgZone, },
            { type: core_1.ChangeDetectorRef, },
            { type: content_1.Content, },
            { type: platform_1.Platform, },
            { type: view_controller_1.ViewController, decorators: [{ type: core_1.Optional },] },
            { type: config_1.Config, },
            { type: dom_controller_1.DomController, },
        ];
        VirtualScroll.propDecorators = {
            '_itmTmp': [{ type: core_1.ContentChild, args: [virtual_item_1.VirtualItem,] },],
            '_hdrTmp': [{ type: core_1.ContentChild, args: [virtual_item_1.VirtualHeader,] },],
            '_ftrTmp': [{ type: core_1.ContentChild, args: [virtual_item_1.VirtualFooter,] },],
            'virtualScroll': [{ type: core_1.Input },],
            'bufferRatio': [{ type: core_1.Input },],
            'approxItemWidth': [{ type: core_1.Input },],
            'approxItemHeight': [{ type: core_1.Input },],
            'approxHeaderWidth': [{ type: core_1.Input },],
            'approxHeaderHeight': [{ type: core_1.Input },],
            'approxFooterWidth': [{ type: core_1.Input },],
            'approxFooterHeight': [{ type: core_1.Input },],
            'headerFn': [{ type: core_1.Input },],
            'footerFn': [{ type: core_1.Input },],
            'virtualTrackBy': [{ type: core_1.Input },],
        };
        return VirtualScroll;
    }());
    exports.VirtualScroll = VirtualScroll;
    var SCROLL_DIFFERENCE_MINIMUM = 20;
});
//# sourceMappingURL=virtual-scroll.js.map